@@ -32,13 +32,11 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 #include <gnuastro/box.h>
 #include <gnuastro/git.h>
 #include <gnuastro/fits.h>
-#include <gnuastro/array.h>
 #include <gnuastro/threads.h>
-#include <gnuastro/txtarray.h>
 #include <gnuastro/statistics.h>
 
-#include <timing.h>
-#include <checkset.h>
+#include <gnuastro-internal/timing.h>
+#include <gnuastro-internal/checkset.h>
 
 #include "main.h"
 
@@ -77,8 +75,8 @@ builtqueue_addempty(struct builtqueue **bq)
   errno=0;
   tbq=malloc(sizeof *tbq);
   if(tbq==NULL)
-    error(EXIT_FAILURE, 0, "%zu byte element in builtqueue_addempty",
-          sizeof *tbq);
+    error(EXIT_FAILURE, 0, "%s: allocating %zu bytes for `tbq'",
+          __func__, sizeof *tbq);
 
   /* Initialize some of the values. */
   tbq->img=NULL;
@@ -119,53 +117,48 @@ saveindividual(struct mkonthread *mkp)
 {
   struct mkprofparams *p=mkp->p;
 
-  size_t len;
   double crpix[2];
+  gal_data_t *data;
   long os=p->oversample;
   struct builtqueue *ibq=mkp->ibq;
-  char *outname, *jobname, *outdir=p->outdir;
+  char *filename, *jobname, *outdir=p->outdir;
 
-  /* Allocate the space for the name of this file. */
-  errno=0;
-  len=strlen(outdir)+NUMBERNAMESTRLEN+strlen(p->mergedimgname);
-  outname=malloc(len*sizeof *outname);
-  if(outname==NULL)
-    error(EXIT_FAILURE, errno, "%zu bytes for name of object in "
-          "row %zu of %s", len*sizeof *outname, ibq->id,
-          p->up.catname);
-
-  /* Save the correct CRPIX values: */
-  crpix[0] = p->crpix[0] - os*(mkp->fpixel_i[0]-1);
-  crpix[1] = p->crpix[1] - os*(mkp->fpixel_i[1]-1);
+  /* Note that `width' is in FITS format, not C. */
+  size_t dsize[2]={mkp->width[1], mkp->width[0]};
 
   /* Write the name and remove a similarly named file. */
-  sprintf(outname, "%s%zu_%s", outdir, ibq->id, p->basename);
-  gal_checkset_check_remove_file(outname, p->cp.dontdelete);
+  asprintf(&filename, "%s%zu_%s", outdir, ibq->id, p->basename);
+  gal_checkset_check_remove_file(filename, 0, p->cp.dontdelete);
 
-  /* Write the array to file (A separately built PSF doesn't need WCS
-     coordinates): */
+  /* Put the array into a data structure */
+  data=gal_data_alloc(ibq->img, GAL_TYPE_FLOAT32, 2, dsize, NULL, 0,
+                      p->cp.minmapsize, "MockImage", "Brightness", NULL);
+
+  /* Write the array to file (a separately built PSF doesn't need WCS
+     coordinates). */
   if(ibq->ispsf && p->psfinimg==0)
-    gal_fits_array_to_file(outname, "MockImg", FLOAT_IMG, ibq->img,
-                           mkp->width[1], mkp->width[0], 0, NULL, NULL,
-                           SPACK_STRING);
+    gal_fits_img_write(data, filename, NULL, PROGRAM_STRING);
   else
-    gal_fits_atof_correct_wcs(outname, "MockImg", FLOAT_IMG, ibq->img,
-                              mkp->width[1], mkp->width[0], p->wcsheader,
-                              p->wcsnkeyrec, crpix, SPACK_STRING);
+    {
+      /* Save the correct CRPIX values: */
+      crpix[0] = p->crpix[0] - os*(mkp->fpixel_i[0]-1);
+      crpix[1] = p->crpix[1] - os*(mkp->fpixel_i[1]-1);
+
+      /* Write the image. */
+      gal_fits_img_write_corr_wcs_str(data, filename, p->wcsheader,
+                                      p->wcsnkeyrec, crpix, NULL,
+                                      PROGRAM_STRING);
+    }
   ibq->indivcreated=1;
 
   /* Report if in verbose mode. */
-  if(p->cp.verb)
+  if(!p->cp.quiet)
     {
-      errno=0;
-      jobname=malloc((len+100)*sizeof *jobname);
-      if(jobname==NULL)
-        error(EXIT_FAILURE, errno, "jobname in mkprof.c");
-      sprintf(jobname, "%s created.", outname);
+      asprintf(&jobname, "%s created.", filename);
       gal_timing_report(NULL, jobname, 2);
       free(jobname);
     }
-  free(outname);
+  free(filename);
 }
 
 
@@ -218,13 +211,12 @@ saveindividual(struct mkonthread *mkp)
    central pixel by one and completely ignore the fractional part.
 */
 void *
-build(void *inparam)
+mkprof_build(void *inparam)
 {
   struct mkonthread *mkp=(struct mkonthread *)inparam;
   struct mkprofparams *p=mkp->p;
 
-  size_t i;
-  double *cat;
+  size_t i, id;
   int lockresult;
   long lpixel_o[2];
   pthread_mutex_t *qlock=&p->qlock;
@@ -232,7 +224,7 @@ build(void *inparam)
   pthread_cond_t *qready=&p->qready;
 
   /* Make each profile that was specified for this thread. */
-  for(i=0;mkp->indexs[i]!=GAL_THREADS_NON_THRD_INDEX;++i)
+  for(i=0; mkp->indexs[i]!=GAL_BLANK_SIZE_T; ++i)
     {
       /* Create a new builtqueue element with all the information. fbq
          will be used when we want to add ibq to p->bq. It is defined
@@ -242,26 +234,25 @@ build(void *inparam)
          fbq->next=p->bq and then set p->bq to ibq.*/
       builtqueue_addempty(&mkp->ibq);
       ibq=mkp->ibq;
-      ibq->id=mkp->indexs[i];
+      id=ibq->id=mkp->indexs[i];
       if(fbq==NULL) fbq=ibq;
-      cat=&p->cat[ibq->id*p->cs1];
 
 
       /* Write the necessary parameters for this profile into mkp.*/
-      setprofparams(mkp);
+      oneprof_set_prof_params(mkp);
 
 
       /* Find the bounding box size (NOT oversampled). */
-      if((int)cat[p->fcol]==POINTCODE)
+      if( p->f[id] == PROFILE_POINT )
         mkp->width[0]=mkp->width[1]=1;
       else
         gal_box_ellipse_in_box(mkp->truncr, mkp->q*mkp->truncr,
-                               cat[p->pcol]*DEGREESTORADIANS, mkp->width);
+                               p->p[id]*DEGREESTORADIANS, mkp->width);
 
 
       /* Get the overlapping pixels using the starting points (NOT
          oversampled). */
-      gal_box_border_from_center(cat[p->xcol], cat[p->ycol], mkp->width,
+      gal_box_border_from_center(p->x[id], p->y[id], mkp->width,
                                  ibq->fpixel_i, ibq->lpixel_i);
       mkp->fpixel_i[0]=ibq->fpixel_i[0];
       mkp->fpixel_i[1]=ibq->fpixel_i[1];
@@ -284,7 +275,7 @@ build(void *inparam)
             gsl_rng_set(mkp->rng, gal_timing_time_based_rng_seed());
 
           /* Make the profile */
-          makeoneprofile(mkp);
+          oneprofile_make(mkp);
           if( p->individual || (ibq->ispsf && p->psfinimg==0))
             {
               saveindividual(mkp);
@@ -310,13 +301,12 @@ build(void *inparam)
               p->bq=ibq;
 
               /* If the list was empty when you locked the mutex, then
-                 either `write` is waiting behind a condition variable
-                 for you to fill it up or not (either it hasn't got to
-                 setting the condition variable yet (this function
-                 locked the mutex before `write`) or it just got the
+                 either `mkprof_write` is waiting behind a condition
+                 variable for you to fill it up or not (either it hasn't
+                 got to setting the condition variable yet (this function
+                 locked the mutex before `mkprof_write`) or it just got the
                  list to be made and is busy writing the arrays in the
-                 output). In either case, pthread_cond_signal will
-                 work. */
+                 output). In either case, pthread_cond_signal will work. */
               if(fbq->next==NULL)
                 pthread_cond_signal(qready);
               pthread_mutex_unlock(qlock);
@@ -326,12 +316,13 @@ build(void *inparam)
               fbq=NULL;
               mkp->ibq=NULL;
             }
+
           /* The mutex couldn't be locked and there are no more
              objects for this thread to build (giving a chance for
              this thread to add up its built profiles). So we have to
              lock the mutex to pass on this built structure to the
              builtqueue. */
-          else if (mkp->indexs[i+1]==GAL_THREADS_NON_THRD_INDEX)
+          else if (mkp->indexs[i+1]==GAL_BLANK_SIZE_T)
             {
               pthread_mutex_lock(qlock);
               fbq->next=p->bq;
@@ -376,66 +367,22 @@ build(void *inparam)
 /************              The writer             *************/
 /**************************************************************/
 void
-writelog(struct mkprofparams *p)
+mkprof_write(struct mkprofparams *p)
 {
-  int space[]={6, 10, 15}, prec[]={3, 6};
-  int int_cols[]={0, 2, 4, -1}, accu_cols[]={-1};
-  char comments[1000], gitdescribe[100], *gd=gal_git_describe();
-
-  if(gd) sprintf(gitdescribe, " from %s,", gd);
-  else   gitdescribe[0]='\0';
-
-  sprintf(comments, "# Log file for "SPACK_STRING".\n"
-          "# Created%s on %s"
-          "# Column 0: Row number in catalog (starting from zero).\n"
-          "# Column 1: Overlap magnitude with final image "
-          "(zeropoint: %.3f).\n"
-          "# Column 2: Number of Monte Carlo integration pixels.\n"
-          "# Column 3: Fraction of brightness in Monte Carlo "
-          "integrated pixels.\n"
-          "# Column 4: An individual image was created.\n",
-          ctime(&p->rawtime), gitdescribe, p->zeropoint);
-
-
-  gal_txtarray_array_to_txt(p->log, p->cs0, LOGNUMCOLS, comments, int_cols,
-                            accu_cols, space, prec, 'f', LOGFILENAME);
-}
-
-
-
-
-
-void
-write(struct mkprofparams *p)
-{
-  void *array;
+  double sum;
   char *jobname;
-  double sum, *log;
   struct timeval t1;
   long os=p->oversample;
   int replace=p->replace;
-  size_t complete=0, cs0=p->cs0;
+  gal_data_t *out=p->out, *log;
   struct builtqueue *ibq=NULL, *tbq;
-  int verb=p->cp.verb, bitpix=p->bitpix;
-  float *out, *to, *from, *colend, *rowend;
+  float *to, *from, *colend, *rowend;
+  size_t complete=0, num=p->num, clog;
   size_t i, j, iw, jw, ii, jj, w=p->naxes[0], ow;
 
 
-  /* Allocate the output array if necessary. */
-  if(p->out)
-    out=p->out;
-  else
-    {
-      errno=0;
-      out=calloc(p->naxes[0]*p->naxes[1], sizeof *out);
-      if(out==NULL)
-        error(EXIT_FAILURE, 0, "%zu bytes for output image",
-              p->naxes[0]*p->naxes[1]*sizeof *out);
-    }
-
-
   /* Write each image into the output array. */
-  while(complete<p->cs0)
+  while(complete<p->num)
     {
       /* Set ibq. */
       if(ibq==NULL)
@@ -458,8 +405,9 @@ write(struct mkprofparams *p)
          that the FITS and C arrays have opposite axis orders and FITS
          counting starts from 1, not zero. Also fpixel is the first
          (inclusive) pixel and so is lpixel (it is inclusive). */
-      if(ibq->overlaps && p->nomerged==0)
+      if(ibq->overlaps && out->array)
         {
+
           /* Set the starting and ending points in the complete image. */
           i  = os * (ibq->fpixel_i[1]-1);
           j  = os * (ibq->fpixel_i[0]-1);
@@ -484,7 +432,7 @@ write(struct mkprofparams *p)
              know the images overlap, iw and jw are both smaller than
              the two image number of columns and number of rows, so
              w-jw and ow-jw will always be positive. */
-          to=out+i*w+j;
+          to = (float *)(out->array) + i*w+j;
           from=ibq->img+ii*ow+jj;
           rowend=to+iw*w;
           do
@@ -512,75 +460,77 @@ write(struct mkprofparams *p)
         }
 
       /* Fill the log array. */
-      log=&p->log[ibq->id*LOGNUMCOLS];
-      log[0] = ibq->id;
-      log[1] = sum>0.0f ? -2.5f*log10(sum)+p->zeropoint : NAN;
-      log[2] = ibq->numaccu;
-      log[3] = ibq->accufrac;
-      log[4] = ibq->indivcreated;
+      if(p->cp.log)
+        {
+          clog=0;
+          for(log=p->log; log!=NULL; log=log->next)
+            switch(++clog)
+              {
+              case 5:
+                ((unsigned char *)(log->array))[ibq->id] = ibq->indivcreated;
+                break;
+              case 4:
+                ((float *)(log->array))[ibq->id] = ibq->accufrac;
+                break;
+              case 3:
+                ((unsigned long *)(log->array))[ibq->id]=ibq->numaccu;
+                break;
+              case 2:
+                ((float *)(log->array))[ibq->id] =
+                  sum>0.0f ? -2.5f*log10(sum)+p->zeropoint : NAN;
+                break;
+              case 1:
+                ((unsigned long *)(log->array))[ibq->id]=ibq->id+1;
+                break;
+              }
+        }
 
       /* Report if in verbose mode. */
       ++complete;
-      if(verb && p->nomerged==0)
+      if(!p->cp.quiet)
         {
-          errno=0;
-          jobname=malloc(100*sizeof *jobname);
-          if(jobname==NULL)
-            error(EXIT_FAILURE, errno, "jobname in mkprof.c");
-          sprintf(jobname, "row %zu complete, %zu left to go",
-                  ibq->id, cs0-complete);
+          asprintf(&jobname, "row %zu complete, %zu left to go",
+                   ibq->id, num-complete);
           gal_timing_report(NULL, jobname, 2);
           free(jobname);
         }
 
-      /* Free the array and the queue element and change it to the
-         next one and increment complete. Note that there is no
-         problem to free a NULL pointer (when the built array didn't
-         overlap). */
+      /* Free the array and the queue element and change it to the next one
+         and increment complete. Note that there is no problem to free a
+         NULL pointer (when the built array didn't overlap). */
       free(ibq->img);
       tbq=ibq->next;
       free(ibq);
       ibq=tbq;
     }
 
-  /* Write the final array to the output FITS image. */
-  if(p->nomerged==0)
+  /* Write the final array to the output FITS image if a merged image is to
+     be created. */
+  if(out->array)
     {
       /* Get the current time for verbose output. */
-      if(verb) gettimeofday(&t1, NULL);
+      if(!p->cp.quiet) gettimeofday(&t1, NULL);
 
-      /* Make a temporary array of the desired type for writing the
-         output. */
-      if(bitpix==FLOAT_IMG)
-        array=out;
-      else
-        gal_fits_change_type(out, FLOAT_IMG, p->naxes[1]*p->naxes[0],
-                             p->anyblank, &array, bitpix);
+      /* Write the final image into a FITS file with the requested type. */
+      gal_fits_img_write_to_type(out, p->mergedimgname, NULL,
+                                 PROGRAM_STRING, p->cp.type);
 
-      /* Write the array to the output FITS file. */
-      gal_fits_array_to_file(p->mergedimgname, "Mock image",
-                             bitpix, array, p->naxes[1],
-                             p->naxes[0], p->anyblank, p->wcs,
-                             NULL, SPACK_STRING);
-
-      /* Free `array' if it was allocated separately.*/
-      if(bitpix!=FLOAT_IMG)
-        free(array);
+      /* Clean up */
+      gal_data_free(out);
 
       /* In verbose mode, print the information. */
-      if(verb)
+      if(!p->cp.quiet)
         {
-          errno=0;
-          jobname=malloc((strlen(p->mergedimgname)+100)*sizeof *jobname);
-          if(jobname==NULL)
-            error(EXIT_FAILURE, errno, "final report in mkprof.c");
-          sprintf(jobname, "%s created.", p->mergedimgname);
+          asprintf(&jobname, "%s created.", p->mergedimgname);
           gal_timing_report(&t1, jobname, 1);
           free(jobname);
         }
     }
 
-  free(out);
+  /* Even with no merged image, there might still be pointers in `out' that
+     need to be freed. */
+  else
+    gal_data_free(p->out);
 }
 
 
@@ -609,11 +559,13 @@ void
 mkprof(struct mkprofparams *p)
 {
   int err;
+  char *tmp;
   pthread_t t;            /* Thread id not used, all are saved here. */
   pthread_attr_t attr;
   pthread_barrier_t b;
   struct mkonthread *mkp;
   size_t i, *indexs, thrdcols;
+  gal_list_str_t *comments=NULL;
   size_t nt=p->cp.numthreads, nb;
   long onaxes[2], os=p->oversample;
 
@@ -624,13 +576,13 @@ mkprof(struct mkprofparams *p)
   errno=0;
   mkp=malloc(nt*sizeof *mkp);
   if(mkp==NULL)
-    error(EXIT_FAILURE, errno,
-          "%zu bytes in mkprof (mkprof.c) for mkp", (nt-1)*sizeof *mkp);
+    error(EXIT_FAILURE, errno, "%s: allocating %zu bytes for `mkp'",
+          __func__, (nt-1)*sizeof *mkp);
 
   /* Distribute the different profiles for different threads. Note
      that one thread is left out for writing, while nt-1 are left
      for building. */
-  gal_threads_dist_in_threads(p->cs0, nt, &indexs, &thrdcols);
+  gal_threads_dist_in_threads(p->num, nt, &indexs, &thrdcols);
 
   /* onaxes are sides of the image without over-sampling. */
   onaxes[0] = (p->naxes[0]-2*p->shift[0])/os + 2*p->shift[0]/os;
@@ -643,26 +595,27 @@ mkprof(struct mkprofparams *p)
       mkp[0].onaxes=onaxes;
       mkp[0].indexs=indexs;
       mkp[0].rng=gsl_rng_clone(p->rng);
-      build(&mkp[0]);
+      mkprof_build(&mkp[0]);
     }
   else
     {
       /* Initialize the attributes. Note that this main thread will
          also have to be kept behind the barrier, so we need nt+1
          barrier stops. */
-      if(p->cs0<nt) nb=p->cs0+1;
+      if(p->num<nt) nb=p->num+1;
       else nb=nt+1;
       gal_threads_attr_barrier_init(&attr, &b, nb);
 
       /* Initialize the condition variable and mutex. */
       err=pthread_mutex_init(&p->qlock, NULL);
-      if(err) error(EXIT_FAILURE, 0, "mutex not initialized");
+      if(err) error(EXIT_FAILURE, 0, "%s: mutex not initialized", __func__);
       err=pthread_cond_init(&p->qready, NULL);
-      if(err) error(EXIT_FAILURE, 0, "condition variable not initialized");
+      if(err) error(EXIT_FAILURE, 0, "%s: condition variable not initialized",
+                    __func__);
 
       /* Spin off the threads: */
       for(i=0;i<nt;++i)
-        if(indexs[i*thrdcols]!=GAL_THREADS_NON_THRD_INDEX)
+        if(indexs[i*thrdcols]!=GAL_BLANK_SIZE_T)
           {
             mkp[i].p=p;
             mkp[i].b=&b;
@@ -670,16 +623,25 @@ mkprof(struct mkprofparams *p)
             mkp[i].onaxes=onaxes;
             mkp[i].rng=gsl_rng_clone(p->rng);
             mkp[i].indexs=&indexs[i*thrdcols];
-            err=pthread_create(&t, &attr, build, &mkp[i]);
+            err=pthread_create(&t, &attr, mkprof_build, &mkp[i]);
             if(err)
-              error(EXIT_FAILURE, 0, "can't create thread %zu", i);
+              error(EXIT_FAILURE, 0, "%s: can't create thread %zu",
+                    __func__, i);
           }
     }
 
   /* Write the created arrays into the image. */
-  write(p);
-  if(p->cp.nolog==0)
-    writelog(p);
+  mkprof_write(p);
+
+  /* Write the log file. */
+  if(p->cp.log)
+    {
+      asprintf(&tmp, "Zeropoint: %g", p->zeropoint);
+      gal_list_str_add(&comments, tmp, 0);
+      gal_table_write_log(p->log, PROGRAM_STRING, &p->rawtime, comments,
+                          LOGFILENAME, p->cp.dontdelete, p->cp.quiet);
+      gal_list_str_free(comments, 1);
+    }
 
   /* If numthreads>1, then wait for all the jobs to finish and destroy
      the attribute and barrier. */
