@@ -19,11 +19,11 @@
 # Preliminaries
 # =============
 #
-# Set the variabels (The executable is in the build tree). Do the
+# Set the variables (The executable is in the build tree). Do the
 # basic checks to see if the executable is made or if the defaults
 # file exists (basicchecks.sh is in the source tree).
 prog=convertt
-img=imgcrop_section.fits
+img=crop_section.fits
 execname=../bin/$prog/ast$prog
 
 
@@ -42,7 +42,10 @@ execname=../bin/$prog/ast$prog
 #     data file failed).
 #
 #   - Ghostscript was not present on the system.
-if [ ! -f $execname ] || [ ! -f $img ] || [ "x$hasghostscript" != "xyes" ]; then
+if [ ! -f $execname ]; then echo "$execname not created."; exit 77; fi
+if [ ! -f $img      ]; then echo "$img does not exist.";   exit 77; fi
+if [ "x$hasghostscript" != "xyes" ]; then
+    echo "Ghostscript 9.10 or above not present.";
     exit 77;
 fi
 
@@ -50,7 +53,6 @@ fi
 
 
 
-
 # Actual test script
 # ==================
-$execname $img --output=pdf --log
+$execname $img --output=pdf --invert
