@@ -19,7 +19,7 @@
 # Preliminaries
 # =============
 #
-# Set the variabels (The executable is in the build tree). Do the
+# Set the variables (The executable is in the build tree). Do the
 # basic checks to see if the executable is made or if the defaults
 # file exists (basicchecks.sh is in the source tree).
 prog=cosmiccal
@@ -36,8 +36,7 @@ execname=../bin/$prog/ast$prog
 # types of dependencies:
 #
 #   - The executable was not made (for example due to a configure option),
-#
-if [ ! -f $execname ]; then exit 77; fi
+if [ ! -f $execname ]; then echo "$execname not created.";    exit 77; fi
 
 
 
