@@ -26,11 +26,11 @@
 # Preliminaries
 # =============
 #
-# Set the variabels (The executable is in the build tree). Do the
+# Set the variables (The executable is in the build tree). Do the
 # basic checks to see if the executable is made or if the defaults
 # file exists (basicchecks.sh is in the source tree).
 prog=mkprof
-execname=../src/$prog/ast$prog
+execname=../bin/$prog/ast$prog
 cat=$topsrc/tests/$prog/mkprofcat2.txt
 
 
@@ -43,9 +43,11 @@ cat=$topsrc/tests/$prog/mkprofcat2.txt
 # If the dependencies of the test don't exist, then skip it. There are two
 # types of dependencies:
 #
-#   - The executable was not made (for example due to a configure option),
+#   - The executable was not made (for example due to a configure option).
 #
-if [ ! -f $execname ]; then exit 77; fi
+#   - Catalog doesn't exist (problem in tarball release).
+if [ ! -f $execname ]; then echo "$execname not created."; exit 77; fi
+if [ ! -f $cat      ]; then echo "$cat does not exist.";   exit 77; fi
 
 
 
